#! /usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Python Version: 3.11.4
Creator: 孙一凡-莫无煜
Create Date: 2023/7/27

"""
from setuptools import setup, find_packages

setup(
    name='movoid_tools',
    version='1.1.7',
    description='mo void create a lot of tools to use in many situations.',
    author='SunYiFan-MoVoid',
    author_email='bobrobotsun@163.com',
    license='MIT',
    packages=find_packages()
)

if __name__ == '__main__':
    pass
